package com.synhaptein.scalator.security

import java.security.MessageDigest

/**
 * HashService component interface.
 *
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright     Copyright 2010-2011, SynHaptein (http://www.synhaptein.com)
 * @link          http://www.synhaptein.com/scalator scalator project
 * @since         scalator 0.2
 * @license       http://www.synhaptein.com/scalator/license.html
 */

trait HashServiceComponent {
  val hashService: HashService

  abstract class HashService(val algorithm: String = "SHA-256")
  {
    private val HEXES = "0123456789abcdef"

    def hashString(string: String): String = {
      val digest = MessageDigest.getInstance(algorithm)
      digest.reset()
      byteArrayToString(digest.digest(addSalt(string).getBytes("UTF-8")))
    }

    protected def addSalt(string: String): String

    private def byteArrayToString(raw: Array[Byte]): String = {
      val hex = new StringBuilder( 2 * raw.length );
      for (val b <- raw) {
        hex.append(HEXES.charAt((b & 0xF0) >> 4))
           .append(HEXES.charAt((b & 0x0F)));
      }
      hex.toString();
    }
  }
}